// node
const fs = require('fs')
const path = require('path')
// npm
const WDIOReporter = require('@wdio/reporter').default

const writeCoverage = function (data) {
  const coverage = browser.execute(function () {
    const coverage = window.__coverage__
    // delete coverage information, so next text has a fresh start
    window.__coverage__ = null
    delete window.__coverage__
    return coverage
  })

  return coverage.then((coverage) => {
    if (coverage) {
      const dir = '.nyc_output'
      fs.mkdirSync(dir, { recursive: true })
      // eslint-disable-next-line no-useless-escape
      let filename = `${data.cid}_${data.uid}_`.replace(/[`~!@#$%^&*|+\-=?;:'",.<>\{\}\[\]\\\/]/gi, '').toLowerCase().replace(/[` ]/gi, '_');

      const targetPath = path.join(dir, path.basename(filename)) + '.json'
      const stream = fs.createWriteStream(targetPath)
      stream.once('open', function () {
        stream.write(JSON.stringify(coverage))
        stream.end()
      })
    }
  })
}

class CustomReporter extends WDIOReporter {
  constructor(options) {
    options = Object.assign(options, { stdout: true })
    super(options)
  }
  onTestFail(data) {
    return writeCoverage(data);
  }
  onTestPass(data) {
    return writeCoverage(data);
  }
}

exports.default = CustomReporter
